#!/usr/bin/env python
# -*- coding: utf-8 -*-
from bigbuild.models import Page
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    help = 'Creates a new page directory'
    INDEX_TEMPLATE_NAME = 'bigbuild/pages/default_index.html'

    def add_arguments(self, parser):
        """
        Custom arguments for this command
        """
        parser.add_argument('slug', nargs='+', type=str)
        parser.add_argument(
            '--force',
            action='store_true',
            dest='force',
            default=False,
            help='Force overwrite of the page directory if it already exists'
        )
        parser.add_argument(
            '--sans',
            action='store_true',
            dest='sans',
            default=False,
            help='Create a page that uses our alternative sans-serif theme'
        )
        parser.add_argument(
            '--dark',
            action='store_true',
            dest='dark',
            default=False,
            help='Create a page that uses our alternative dark theme'
        )

    def set_options(self, *args, **options):
        """
        Configures the command based on user input.
        """
        self.force = options['force']
        self.sans = options['sans']
        self.dark = options['dark']
        self.slug_list = options['slug']

    def create_page(self, slug):
        """
        Returns a new Page object given the slug submitted by the user.
        """
        return Page(slug)

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        # Set the options
        self.set_options(*args, **options)

        # Loop through the slugs and create pages
        for slug in self.slug_list:

            # Create the Page
            page = self.create_page(options['slug'])

            # If the directory already exists and we're not forcing creation
            # an error should be thrown.
            if (
                page.directory_exists or page.retired_directory_exists
            ) and not self.force:
                    raise CommandError(
                        self.style.ERROR('Page directory already exists')
                    )
            # Otherwise we make the page
            else:
                page.create_directory(
                    force=self.force,
                    index_template_name=self.INDEX_TEMPLATE_NAME,
                    index_template_context=dict(
                        sans=self.sans,
                        dark=self.dark,
                    )
                )

            # Print out the result
            self.stdout.write(self.style.SUCCESS('Created page "%s"' % page))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from django import template
from bigbuild.models import Page
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape
register = template.Library()


@register.filter(needs_autoescape=True)
def dropcap(text, autoescape=True):
    """
    Wraps the first character in a <span> tag with a .dropcap class
    that can be used to make it bigger than the surrounding text.
    """
    if not text:
        return ''
    first, other = text[0], text[1:]
    if autoescape:
        esc = conditional_escape
    else:
        def esc(x): return x
    template = "<span class='dropcap'>{}</span>{}"
    html = template.format(esc(first.upper()), esc(other))
    return mark_safe(html)


@register.filter
def jsonify(obj):
    """
    Returns an object in JSON format.
    """
    if isinstance(obj, Page):
        return json.dumps(obj.to_json())
    return json.dumps(obj)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom exceptions and warnings
"""
from django.template import Engine, Context
from django.utils.termcolors import colorize


class BadMetadata(Exception):
    """
    A custom exception to raise when the page configuration file has
    malformed metadata.
    """
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)


class BaseWarning(Warning):
    """
    A base Warning class with elements we want to reuse across all of
    our custom warnings.
    """
    BANNER_TEMPLATE = "bigbuild/warnings/banner.txt"
    FOOTER_TEMPLATE = "bigbuild/warnings/footer.txt"
    BOTTOM_BORDER = "bigbuild/warnings/bottom_border.txt"

    def render_template(self, name, context={}):
        """
        Returns the rendered template with the provided name.
        """
        engine = Engine.get_default()
        return engine.get_template(name).render(Context(context))

    def hilight(self, s):
        """
        Returns a colorized version of the provided string that is red
        and bold.
        """
        return colorize(s, fg="red", opts=('bold',))

    @property
    def banner(self):
        """
        Returns the top of every warning message
        """
        message = self.render_template(self.BANNER_TEMPLATE)
        return self.hilight(message)

    @property
    def footer(self):
        """
        Returns the bottom of every warning message
        """
        footer_message = self.render_template(self.FOOTER_TEMPLATE)
        border_message = self.render_template(self.BOTTOM_BORDER)
        return footer_message + self.hilight(border_message)

    def get_context_data(self):
        """
        Returns the data to be passed into the message template as context.
        """
        return {}

    def __str__(self):
        """
        Returns the full message for this warning, which will be what's
        printed to the console when it is logged.
        """
        message = self.render_template(
            self.template_name,
            self.get_context_data()
        )
        return self.banner + message + self.footer


class MissingMetadataWarning(BaseWarning):
    """
    A custom warning to broadcast when a page directory does not
    have a metadata.md configuration file.
    """
    template_name = "bigbuild/warnings/missing_metadata.txt"

    def __init__(self, page):
        self.page = page

    def get_context_data(self):
        return dict(page=self.page)


class MissingRecommendedMetadataWarning(BaseWarning):
    """
    A custom warning that will be raised when a published page is missing
    recommended metadata values.

    For instance, things like a headline and description are
    not required to publish a page, but you're likely to have bugs
    with how your page appears on social media if you lack them.
    """
    template_name = "bigbuild/warnings/missing_recommended_metadata.txt"

    def __init__(self, page):
        self.page = page

    def get_context_data(self):
        return dict(page=self.page)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import csv
import json
import yaml
import codecs
import shutil
import logging
import frontmatter
from bigbuild.models import BasePage
from bigbuild.exceptions import BadMetadata
from django.template import Engine, Context
from bigbuild import get_page_directory, get_retired_directory
logger = logging.getLogger(__name__)


class Page(BasePage):
    """
    A custom page published via static.latimes.com
    """
    @property
    def base_directory(self):
        """
        Returns base directory for pages.
        """
        return get_page_directory()

    @property
    def retired_directory_path(self):
        """
        Returns the path where this page would be retired, if it were retired.
        """
        return os.path.join(get_retired_directory(), self.slug)

    @property
    def retired_directory_exists(self):
        """
        Tests whether a retired directory for this slug already exists.
        """
        return os.path.exists(self.retired_directory_path)

    def create_directory(
        self,
        force=False,
        index_template_name='bigbuild/pages/default_index.html',
        index_template_context={},
    ):
        """
        Creates a new directory for the page.

        Returns the path to the directory that has been created, which is
        the same as the self.directory_path property.

        Throws an error if the directory already exists. You can force it
        to overwrite a pre-existing directory by submitting the force keyword
        argument as true.
        """
        # Check if the directory exists already and act accordingly
        if self.directory_exists:
            if force:
                shutil.rmtree(self.directory_path)
            else:
                raise ValueError(
                    "Page directory already exists at %s" % self.directory_path
                )

        # Make the directory
        os.mkdir(self.directory_path)

        # Add the index template
        self.write_index(
            template_name=index_template_name,
            template_context=index_template_context,
        )

        # Create the metadata configuration file
        self.write_frontmatter()

        # Create the ./static/ directory for extra stuff
        self.write_static()

        # Create the checklist file
        self.write_checklist()

        # Return the directory path
        return self.directory_path

    def write_index(self, template_name, template_context={}):
        """
        Creates index.html in the page directory.
        """
        template = self.get_template(template_name)
        html = template.render(Context(template_context))
        with codecs.open(self.index_path, 'w', 'utf-8') as f:
            f.write(html)

    def write_frontmatter(self, path=None):
        """
        Creates metadata.yaml in the page directory, or a supplied target directory.
        """
        frontmatter.dump(self, path or self.frontmatter_path)

    def write_static(self):
        """
        Creates a ./static/ subdirectory within the page directory.
        """
        os.mkdir(self.static_path)

        css_path = os.path.join(self.static_path, 'style.css')
        css = self.get_template("bigbuild/pages/style.css-tpl").source
        with codecs.open(css_path, 'w', 'utf-8') as f:
            f.write(css)

        js_path = os.path.join(self.static_path, 'app.js')
        js = self.get_template("bigbuild/pages/app.js-tpl").source
        with codecs.open(js_path, 'w', 'utf-8') as f:
            f.write(js)

    def write_checklist(self):
        """
        Creates checklist.md in the page directory.
        """
        template = self.get_template("bigbuild/pages/checklist.md-tpl")
        md = template.render(Context(dict(object=self)))
        with codecs.open(self.checklist_path, 'w', 'utf-8') as f:
            f.write(md)

    def get_template(self, name):
        """
        Returns a Django template of the provided name ready to render.
        """
        engine = Engine.get_default()
        return engine.get_template(name)

    def sync_frontmatter(self):
        super(Page, self).sync_frontmatter()
        with codecs.open(self.frontmatter_path) as f:
            # Parse the YAML with confidence since the super call above
            # has already weeded out any errors
            post = frontmatter.load(f)
            # Loop through any data files
            self.data = {}
            for key, path in post.metadata.get('data', {}).items():
                # Generate what the path to the file ought to be
                p = os.path.join(self.data_path, path)
                # Toss an error if it doesn't exist
                if not os.path.exists(p):
                    raise BadMetadata("Data file could not be found at %s" % p)
                # Open the file
                with codecs.open(p, 'r') as f:
                    # If it's a CSV file open it that way...
                    if p.endswith(".csv"):
                        self.data[key] = list(csv.DictReader(f))
                    # If it's a JSON file open it this way ...
                    elif p.endswith(".json"):
                        self.data[key] = json.load(f)
                    # If it's a YAML file open it t'other way ...
                    elif (p.endswith(".yml") or p.endswith(".yaml")):
                        self.data[key] = yaml.load(f)
                    # If it's none of those throw an error.
                    else:
                        raise BadMetadata(
                            "Data file at %s not recognizable type" % path
                        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .base import BasePage
from .pages import Page
from .retiredpages import RetiredPage
from .pagelists import PageList


__all__ = (
    'Page',
    'BasePage',
    'PageList',
    'RetiredPage',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from bakery.management.commands.build import Command as Build


class Command(Build):
    """
    An override of Django's base `build` command that changes some of the
    options to defaults we'd prefer here on this site.
    """
    verbosity = 1

    def handle(self, *args, **options):
        # Cut out some of the bakery defaults we don't want
        options['skip_static'] = True
        options['skip_media'] = True

        # Run the standard bakery build
        super(Command, self).handle(*args, **options)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf import settings


def environment(request):
    """
    Adds env-related context variables to the context.
    """
    return {
        'DEVELOPMENT': settings.DEVELOPMENT,
        'STAGING': settings.STAGING,
        'PRODUCTION': settings.PRODUCTION,
        'BIGBAR': settings.BIGBAR
    }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Slimmed down subclasses of django-compressor internals that aim to work
without django.contrib.staticfiles installed.
"""
import codecs
from django.conf import settings
from bigbuild import get_base_url
from compressor.base import Compressor
from compressor.js import JsCompressor
from compressor.css import CssCompressor
from django.template import Template, Context
from compressor.exceptions import UncompressableFileError


class SimpleCompressor(Compressor):
    """
    A simplification of django-compressor's standard compression class.

    The aim is to work without django.contrib.staticfiles installed.
    """
    def get_basename(self, url):
        """
        Takes full path to a static file (eg. "/static/css/style.css") and
        returns path with storage's base url removed (eg. "css/style.css").
        """
        base_url = get_base_url()
        if not url.startswith(base_url):
            raise UncompressableFileError("'%s' isn't accessible via "
                                          "COMPRESS_URL ('%s') and can't be "
                                          "compressed" % (url, base_url))
        basename = url.replace(base_url, "", 1)
        # drop the querystring, which is used for non-compressed cache-busting.
        return basename.split("?", 1)[0]

    def get_filename(self, basename):
        """
        A simplification of the standard method to remove a hack around
        the staticfiles caching system we don't want.
        """
        # Get the filename from our storage backend
        filename = self.storage.path(basename)

        # If it exists, return it
        if self.storage.exists(basename):
            return filename

        # If it doesn't, raise an exception
        raise UncompressableFileError(
            "'%s' could not be found in the COMPRESS_ROOT '%s'%s" %
            (basename, settings.COMPRESS_ROOT,
             self.finders and " or with staticfiles." or "."))

    def precompile(
        self,
        content,
        kind=None,
        elem=None,
        filename=None,
        charset=None,
        **kwargs
    ):
        """
        An expansion of the standard method that will halt precompilation
        when compression is off.
        """
        # If compression is off, skip it
        if not kind or not settings.COMPRESS_ENABLED:
            return False, content

        # Clear out the filename settings so the compilers will use
        # the rendered in-memory string and a temporary file instead.
        kwargs.pop("basename", "")
        filename = None

        # Otherwise compile away as usual
        return super(SimpleCompressor, self).precompile(
            content,
            kind,
            elem,
            filename,
            charset,
            **kwargs
        )

    def get_filecontent(self, filename, charset):
        """
        A custom override that renders file content as a Django template
        with the page context included.

        This allows for metadata from the page object to be included in
        static files.
        """
        if charset == 'utf-8':
            # Removes BOM
            charset = 'utf-8-sig'
        with codecs.open(filename, 'r', charset) as fd:
            # All the custom bits are right here
            content = fd.read()
            template = Template(content)
            context = Context(self.context)
            rendered_content = template.render(context)
            return rendered_content


class SimpleCssCompressor(CssCompressor, SimpleCompressor):
    """
    Our custom CSS compressor.
    """
    def __init__(self, content=None, output_prefix="css", context=None):
        """
        Override that introduces a new setting, COMPRESS_CSS_ENABLED, that lets only CSS compression be turned off.
        """
        if getattr(settings, 'COMPRESS_CSS_ENABLED', True):
            filters = list(settings.COMPRESS_CSS_FILTERS)
        else:
            filters = list()
        super(CssCompressor, self).__init__(content, output_prefix, context, filters)


class SimpleJsCompressor(JsCompressor, SimpleCompressor):
    """
    Our custom JavaScript compressor.
    """
    def __init__(self, content=None, output_prefix="js", context=None):
        """
        Override that introduces a new setting, COMPRESS_HS_ENABLED, that lets only JS compression be turned off.
        """
        if getattr(settings, 'COMPRESS_JS_ENABLED', True):
            filters = list(settings.COMPRESS_JS_FILTERS)
        else:
            filters = []
        super(JsCompressor, self).__init__(content, output_prefix, context, filters)



import os
import logging
import tempfile
from datetime import datetime
from bigbuild import exceptions
from greeking import latimes_ipsum
from bigbuild.models import PageList, Page
from django.core.management import call_command
from bigbuild.compressors import SimpleCompressor
from django.core.management.base import CommandError
from bigbuild.templatetags.bigbuild_tags import dropcap
from bigbuild.templatetags.bigbuild_tags import jsonify
from bigbuild.views import PageListView, PageDetailView
from compressor.exceptions import UncompressableFileError
from django.test import SimpleTestCase, override_settings
logging.disable(logging.CRITICAL)

TEMP_DIR = tempfile.mkdtemp()
BUILD_DIR = os.path.join(TEMP_DIR, '.build')
PAGE_DIR = os.path.join(TEMP_DIR, '.pages')
RETIRED_DIR = os.path.join(TEMP_DIR, '.retired')
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [PAGE_DIR, RETIRED_DIR],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [],
        },
    },
]


@override_settings(RETIRED_DIR=RETIRED_DIR)
@override_settings(BUILD_DIR=BUILD_DIR)
@override_settings(PAGE_DIR=PAGE_DIR)
@override_settings(TEMPLATES=TEMPLATES)
@override_settings(COMPRESS_ROOT=PAGE_DIR)
class FakePagesTest(SimpleTestCase):
    """
    Tests that run against fake pages.
    """
    @classmethod
    def setUpClass(cls):
        super(FakePagesTest, cls).setUpClass()
        pages = [
            Page(slug='My Fake Page', published=True),
            Page(
                slug='My Second Fake Page',
                published=True,
                description="TK"
            ),
            Page(
                slug='a-unpublished-fake-page',
                published=False,
                description="TK"
            ),
            Page(
                slug='a-future-fake-page',
                published=True,
                pub_date=datetime(2030, 1, 1, 0, 0, 0),
                description=''
            ),
            Page(
                slug='a-working-page',
                published=False,
                description='Foobar'
            ),
            Page(
                slug='a-live-page',
                published=True,
                pub_date=datetime(2001, 1, 1, 0, 0, 0),
                description='Foobar'
            )
        ]
        [p.create_directory() for p in pages]
        pages[0].create_directory(force=True)
        call_command("retirepage", pages[0].slug)
        # Create a blacklisted file to test that
        with open(os.path.join(PAGE_DIR, '.DS_Store'), 'w+') as f:
            f.write("foo")

    @override_settings(BAKERY_GZIP=True)
    def test_gzip(self):
        call_command("build")

    def test_page(self):
        obj = PageList()[0]
        obj.__str__()
        obj.__repr__()
        obj.get_absolute_url()

        self.assertTrue(obj.directory_exists)
        self.assertFalse(obj.retired_directory_exists)

        with self.assertRaises(ValueError):
            obj.create_directory()
        self.assertTrue(obj.is_metadata_valid())

        obj.image_url = 100
        self.assertFalse(obj.is_metadata_valid())

        obj.image_url = ''
        obj.pub_date = 'foobar'
        self.assertFalse(obj.is_metadata_valid())

        obj.image_url = 'http://www.foobar.com'
        obj.pub_date = datetime.now()
        self.assertTrue(obj.is_metadata_valid())

        obj.headline = latimes_ipsum.get_story().headline
        obj.published = True
        self.assertFalse(obj.has_recommended_metadata())

        obj.byline = "foobar"
        obj.headline = "something else"
        self.assertTrue(obj.has_recommended_metadata())

    def test_retiredpage(self):
        p = PageList()['my-fake-page']
        p.__str__()
        p.__repr__()
        p.get_absolute_url()
        p.directory_path
        p.metadata
        PageDetailView().build_object(p)

    def test_pagelist(self):
        with override_settings(PAGE_PUBLICATION_STATUS='working'):
            page_list = PageList()
            self.assertTrue(len(page_list) > 0)
        with override_settings(PAGE_PUBLICATION_STATUS='live'):
            page_list = PageList()
            self.assertTrue(len(page_list) > 0)
        page_list = PageList()
        with self.assertRaises(KeyError):
            page_list['foobar']
        with self.assertRaises(IndexError):
            page_list[100]

    def test_views(self):
        PageListView.as_view()
        PageDetailView.as_view()

    def test_build(self):
        call_command("build")
        self.assertTrue(os.path.exists(BUILD_DIR))
        self.assertTrue(os.path.exists(
            os.path.join(
                BUILD_DIR,
                PageList()['my-fake-page'].get_absolute_url()[1:],
                'index.html'
            )
        ))
        PageList()[0].build()

    def test_validatepages(self):
        call_command("validatepages")
        p = Page(slug='my-invalid-page', pub_date=1)
        p.create_directory()
        with self.assertRaises(ValueError):
            call_command("validatepages")

    def test_createpage(self):
        call_command("createpage", "test-page")
        with self.assertRaises(CommandError):
            call_command("createpage", "test-page")

    def test_createbigbuild(self):
        call_command("createbigbuild", "test-bigbuild")

    def test_creategraphic(self):
        call_command("creategraphic", "test-graphic")

    def test_retirepage(self):
        p = Page(slug='test-retired-page', published=True, description="TK")
        p.create_directory()
        call_command("retirepage", p.slug)
        with self.assertRaises(CommandError):
            call_command("retirepage", p.slug)
        with self.assertRaises(CommandError):
            call_command("retirepage", "hello-wtf")

    def test_warnings(self):
        exceptions.BadMetadata()
        exceptions.BaseWarning().get_context_data()
        exceptions.MissingMetadataWarning('slug').__str__()
        exceptions.MissingRecommendedMetadataWarning('slug').__str__()

    def test_malformed_yaml(self):
        p = Page(slug='test-bad-yaml', published=True)
        p.create_directory()

        yaml = open(p.frontmatter_path).read()

        bad = open(p.frontmatter_path, 'w')
        bad.write("""
---
foo:: bar;:
---
""")
        bad.close()

        with self.assertRaises(exceptions.BadMetadata):
            p.sync_frontmatter()

        good = open(p.frontmatter_path, 'w')
        good.write(yaml)
        good.close()

    def test_delete(self):
        p = PageList()[0]
        p.delete()

    def test_missingmetadata(self):
        p = Page(slug='test-no-yaml', published=True)
        p.create_directory()
        os.remove(p.frontmatter_path)
        PageList()
        p.delete()

    def test_data(self):
        p = PageList()['my-second-fake-page']

        p.data = {"foo": "bar.csv"}
        p.write_frontmatter()
        with open(os.path.join(p.data_path, 'bar.csv'), 'w+') as f:
            f.write('foo,bar')
        p.sync_frontmatter()
        p.metadata['data']['foo']

        p.data = {"foo": "bar.json"}
        p.write_frontmatter()
        with open(os.path.join(p.data_path, 'bar.json'), 'w+') as f:
            f.write('[{"foo": "bar"}]')
        p.sync_frontmatter()
        p.metadata['data']['foo']

        p.data = {"foo": "bar.yml"}
        p.write_frontmatter()
        with open(os.path.join(p.data_path, 'bar.yml'), 'w+') as f:
            f.write('- foo: bar')
        p.sync_frontmatter()
        p.metadata['data']['foo']

    def test_baddata(self):
        p = PageList()['my-second-fake-page']

        p.data = {"foo": "bar.csv"}
        p.write_frontmatter()
        with self.assertRaises(exceptions.BadMetadata):
            p.sync_frontmatter()
        p.data = {}
        p.write_frontmatter()

        os.path.exists(p.data_path) or os.mkdir(p.data_path)
        with open(os.path.join(p.data_path, 'foo.txt'), 'w+') as f:
            f.write("foo,bar")
        p.data = {"foo": "foo.txt"}
        p.write_frontmatter()
        with self.assertRaises(exceptions.BadMetadata):
            p.sync_frontmatter()
        p.data = {}
        p.write_frontmatter()

    def test_sans(self):
        p = Page(slug='test-sans', published=True)
        p.create_directory(index_template_context={'sans': True})

    def test_dark(self):
        p = Page(slug='test-dark', published=True)
        p.create_directory(index_template_context={'dark': True})

    def test_nofile(self):
        """
        Test django-compressor errors
        """
        c = SimpleCompressor()
        with self.assertRaises(UncompressableFileError):
            c.get_filename("foobar.jpg")

    def test_nocompress(self):
        """
        Test compression turned off
        """
        with override_settings(COMPRESS_ENABLED=False):
            call_command("build")


class DropCapTest(SimpleTestCase):
    """
    Tests for the dropcap template filter
    """
    def test_dropcap(self):
        self.assertEqual(
            dropcap("Foo bar"),
            "<span class='dropcap'>F</span>oo bar"
        )
        self.assertEqual(
            dropcap("Foo bar", autoescape=False),
            "<span class='dropcap'>F</span>oo bar"
        )
        self.assertEqual(dropcap(""), "")


class JsonifyTest(SimpleTestCase):
    """
    Test the jsonify template filter
    """
    def test_jsonify(self):
        p = Page(slug='test-sans', published=True)
        jsonify(p)
        jsonify(dict(foo="bar"))



import os
from git import Repo
from django.conf import settings
default_app_config = 'bigbuild.apps.BigbuildConfig'


def get_page_directory():
    """
    Returns the PAGE_DIR where dynamic pages are configured.
    """
    # Return the PAGE_DIR settings, if it's been set, or fall back to the default.
    path = getattr(
        settings,
        'PAGE_DIR',
        os.path.join(settings.BASE_DIR, 'pages')
    )
    # Make the directory if it doesn't exist already
    os.path.exists(path) or os.mkdir(path)
    # Return the path
    return path


def get_retired_directory():
    """
    Returns the RETIRED_DIR where retired pages are configured.
    """
    # Return the RETIRED_DIR settings, if it's been set, or fall back to the default.
    path = getattr(
        settings,
        'RETIRED_DIR',
        os.path.join(settings.BASE_DIR, '.retired')
    )
    # Make the directory if it doesn't exist already
    os.path.exists(path) or os.mkdir(path)
    # Return the path
    return path


def get_repo_branch():
    """
    Returns the name of the current git branch.
    """
    if getattr(settings, 'BIGBUILD_GIT_BRANCH', None):
        return settings.BIGBUILD_GIT_BRANCH
    repo_path = getattr(settings, 'BIGBUILD_GIT_DIR', settings.BASE_DIR)
    branch = Repo(repo_path).active_branch
    return branch.name


def get_base_url():
    """
    Returns the base URL for site.
    """
    base_url = getattr(settings, 'BIGBUILD_BASE_URL', '')
    if base_url.startswith("/"):
        base_url = base_url[1:]

    repo_branch = get_repo_branch()

    if settings.DEBUG:
        base_url = os.path.join(
            "/",
            repo_branch,
            base_url
        )
    else:
        base_url = os.path.join(
            "/",
            base_url,
        )

    return base_url



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from bigbuild.models import BasePage
from bigbuild import get_retired_directory


class RetiredPage(BasePage):
    """
    A retired custom page published via static.latimes.com
    """
    @property
    def base_directory(self):
        """
        Returns retired directory for pages.
        """
        return get_retired_directory()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import six
import logging
from django.conf import settings
from collections import Sequence
from bigbuild.exceptions import (
    MissingMetadataWarning,
    MissingRecommendedMetadataWarning
)
from bigbuild.models import Page, RetiredPage
from bigbuild import get_page_directory, get_retired_directory
logger = logging.getLogger(__name__)


class PageList(Sequence):
    """
    A list of all the Page objects in the application.
    """
    def __init__(self):
        # Set the page directories
        self.dynamic_directory = get_page_directory()
        self.retired_directory = get_retired_directory()
        # Create a combined list of live pages and retired pages
        self.pages = []
        self.pages.extend(self.get_dynamic_pages())
        self.pages.extend(self.get_retired_pages())
        # Sort first by reverse chron
        self.pages = sorted(
            self.pages,
            key=lambda o: o.pub_date,
            reverse=True
        )
        # Then sort the working pages to the top
        self.pages = sorted(self.pages, key=lambda o: o.is_live())

    def __iter__(self):
        """
        What to use when this PageList is used in a for loop.
        """
        return iter(self.pages)

    def __len__(self):
        """
        Returns the number of pages in this PageList.
        """
        return len(self.pages)

    def __getitem__(self, key):
        """
        Pull items either with a slug key or an integer index.
        """
        if isinstance(key, six.string_types):
            try:
                return [i for i in self.pages if i.slug == key][0]
            except IndexError:
                raise KeyError("No page with that slug was found")
        elif isinstance(key, int):
            try:
                return self.pages[key]
            except IndexError:
                raise IndexError("No page with this key could be found")

    def get_page_list(self, path, pagetype):
        """
        Returns a list of Page objects from the provided directory path.
        """
        page_list = []
        # Loop through all of the directories in the page directory
        for d in os.listdir(path):
            # Ignore any names in our blacklist
            if d in getattr(settings, 'PAGE_BLACKLIST', ['.DS_Store']):
                continue

            # Create a Page object from the directory slug
            page = pagetype(d)

            # Verify it has frontmatter and is a qualified page
            if not os.path.exists(page.frontmatter_path):
                # If it doesn't have frontmatter broadcast a warning
                # to the developer and skip this directory.
                warning = MissingMetadataWarning(page)
                logger.warn(warning)
                continue

            # Sync in the metadata from the filesystem to the object
            page.sync_frontmatter()

            # Make sure the page is valid
            if not page.is_metadata_valid():
                raise ValueError("Metadata is not valid for %s" % page)

            # Make sure the page has recommended metadata
            # ... if it's ready to publish
            if page.pub_status in ['live', 'pending']:
                if not page.has_recommended_metadata():
                    logger.warn(MissingRecommendedMetadataWarning(page))

            page_list.append(page)

        # Return the page_list
        return page_list

    def get_dynamic_pages(self):
        """
        Returns a list of Page objects ready to be built
        in this environment.
        """
        return [
            p for p in self.get_page_list(self.dynamic_directory, Page)
            if p.should_build()
        ]

    def get_retired_pages(self):
        """
        Returns a list of RetiredPage objects ready to be built
        in this environment.
        """
        return [
            p for p in self.get_page_list(self.retired_directory, RetiredPage)
            if p.should_build()
        ]
